//import terrapin.*;
import processing.pdf.*;

int XSIZE=int(3.6*297);
int YSIZE=int(3.6*210);

PFont font16, font32;
PGraphicsPDF pdf;

void setup() {
     noLoop();
     size(XSIZE,YSIZE);
     pdf = (PGraphicsPDF) beginRecord(PDF, "CCC_Nr3_FLOAT.pdf");
    //this was tricky, see JunTestFont or email 20-12-2011
}

class PPoint extends PVector {
      PPoint(float x, float y){
            super(x,y);
      }
      //the following method does not change "this" but makes a new point
      //I would have preferred a static method,  but that was not allowed
      //inpol interpolates "this" and B, to give a middle point, half way 
      PPoint intpol(PPoint B){
             PVector M = mult(PPoint.add(this,B),0.5);
             return new PPoint(M.x,M.y);
      }
      PPoint intpol(PPoint B, float f){
             PVector M = PPoint.add(mult(this,1-f),mult(B,f));
             return new PPoint(M.x,M.y);
      }
}

void setPosition(Terrapin t, PPoint P){
     t.x = (P.x);
     t.y = (P.y);
}
PPoint getPosition(Terrapin t){
     return new PPoint(t.x,t.y);
}
void setRotation(Terrapin t, float theta){
     t.setRotation(theta);
}
float getRotation(Terrapin t){
      return int(t.getRotation());
}
   
class CCC {
      Terrapin t;
      //designed for vertical 
      //glide reflection axis only
      PPoint A,B,C,M1,M2,M3;
      //to store the direction of the arrow
      //run ABC before the twoversions method
      float a2m;
      //the following variables can be used after the figure-construction
      //method ABCMfourversions has done its work, useful for finding translation vectors;
      PPoint A1,B1,C1,M11,M21,M31,A2,B2,C2,M12,M22,M32;
      CCC(Terrapin trp){
         t=trp;
      }
void ABC(PPoint inipos,int inidir) {
        // this method uses one arbitrary line A2M and one almost arbitrary line C2M;
        // the construction method uses guarantees a verical axis of glide reflection 
        // when inidir = 0, that is the positive x axis; the procedure encodes the logic
        // of how the lines are positioned with respect to the four special points A,B,C,M.
        setPosition(t,inipos);
        setRotation(t,inidir);
        int s = 24; //scale, good values are 20, 24
        A = getPosition(t);
        //create AB arbitrarily
        A2M1(t,s);
        a2m = getRotation(t); 
        M1 = getPosition(t);
        B = new PPoint(M1.x,M1.y); 
        B.add(PVector.sub(M1,A));
        setPosition(t,B);
        setRotation(t,inidir);
        t.left(180);   
        A2M1(t,s);
        //create AC arbitrarily
        setPosition(t,A);
        setRotation(t,inidir);
        A2M3(t,s);
        M3 = getPosition(t);
        C = new PPoint(M3.x,M3.y); 
        C.add(PVector.sub(M3,A));
        setPosition(t,C);
        setRotation(t,inidir);
        t.left(180);   
        A2M3(t,s);
        //create CB making sure CM2 meets M2 being the middle of BC 
        setPosition(t,C);
        t.setRotation(inidir);
        M2 = C.intpol(B);
        C2M2(t,s,M2);
        setPosition(t,B);
        setRotation(t,inidir);
        t.left(180);
        C2M2(t,s,M2);
        //finally add a nice arrow
        setPosition(t,C.intpol(M1,.45));
        setRotation(t,a2m);
        t.right(90);
        drwArrowFull(t,s);
        //done
        drwA(A,s);
        drwB(B,s);
        drwC(C,s);
        drwM1(M1,s);
        drwM2(M2,s);
        drwM3(M3,s);

      }//end ABC
      
void ABCtwoversions(PPoint inipos,int inidir) {
        //this procedure is almost like two versions of ABC;
        //but things are scaled down, and special care is taken
        //not to draw any double line; Each figure is numbered so\
        //the first has points A1,B1,C1,M11,M21,M31 the next A2,B2,C2,M12,M22,M32.
        //of course some points coincide, for example A2 = B1.
        //the construction works from right to left.
        setPosition(t,inipos);
        setRotation(t,inidir);
        int s = 8; //scale, good values are 5,8,10, or 20
        //make first ABC
        A1 = getPosition(t);
        A2M1(t,s);
        M11 = getPosition(t);
        B1 = new PPoint(M11.x,M11.y); 
        B1.add(PVector.sub(M11,A1));
        setPosition(t,B1);
        setRotation(t,inidir);
        t.left(180);   
        A2M1(t,s);
        setPosition(t,A1);
        setRotation(t,inidir);
        A2M3(t,s);
        M31 = getPosition(t);
        C1 = new PPoint(M31.x,M31.y); 
        C1.add(PVector.sub(M31,A1));
        setPosition(t,C1);
        setRotation(t,inidir);
        t.left(180);   
        A2M3(t,s);
        setPosition(t,C1);
        t.setRotation(inidir);
        M21 = C1.intpol(B1);
        C2M2(t,s,M21);
        setPosition(t,B1);
        setRotation(t,inidir);
        t.left(180);
        C2M2(t,s,M21);
        //add a nice arrow
        setPosition(t,C1.intpol(M11,.45));
        setRotation(t,a2m);
        t.right(90);
        drwArrowFull(t,s);
        //make ABC version 2
        A2 = B1;
        B2 = A1;
        setPosition(t,A2);
        setRotation(t,inidir);
        t.left(180);
        A2M3(t,s);
        M32 = getPosition(t);
        C2 = new PPoint(M32.x,M32.y); 
        C2.add(PVector.sub(M32,A2));
        setPosition(t,C2);
        setRotation(t,inidir);   
        A2M3(t,s);
        setPosition(t,C2);
        t.setRotation(inidir);
        t.left(180);
        M22 = C2.intpol(B2);
        C2M2(t,s,M22);
        setPosition(t,B2);
        setRotation(t,inidir);
        C2M2(t,s,M22);
        //and another nice arrow
        setPosition(t,C2.intpol(M11,.45));
        setRotation(t,a2m);
        t.left(90);
        drwArrowFull(t,s);
        drwA(A1,s);
        drwB(B1,s);
        drwC(C1,s);
        drwM1(M11,s);
        drwM2(M21,s);
        drwM3(M31,s);
        drwA(A2,s);
        drwB(B2,s);
        drwC(C2,s);
        drwM2(M22,s);
        drwM3(M32,s);
      }//end ABCtwoversions
}// end CCC

void draw() {
     fill(0);
     stroke(0);
     background(255);
     pdf.textMode(SHAPE);
     Terrapin t = new Terrapin(this);
     font16 = createFont("Comic Sans MS",16); 
     font32 = createFont("Comic Sans MS",16); 
     drwIntro();
     CCC ccc = new CCC(t);
     
     //create one big basic figure
     ccc.ABC(new PPoint(250,280),0);
     
     //create one smaller version
     PPoint origin = new PPoint(700,280);
     ccc.ABCtwoversions(origin,0);
     
     //from this one derive the translation vectors
     //for the entire two-block and repeat by translations;
     //sorry for the double lines;
     PVector T1 = PVector.sub(ccc.B1,ccc.A1);
     PVector T2 = PVector.sub(ccc.C1,ccc.A1);
     for (int i=0;i<3;i++){
         for (int j=0;j<2;j++){
             PPoint T = new PPoint(origin.x,origin.y);
             T.add(PVector.mult(T1,i));
             T.add(PVector.mult(T2,j));
             ccc.ABCtwoversions(T,0);
         }
     }
     endRecord();
}

void A2M1(Terrapin t, int scalefactor){
     //a kind of puzzle-piece line, very arbitrary;
     //make sure not to use any absolute coordinates
     t.setPenColor(0, 0, 0);
     int d=scalefactor;
     strokeWeight(d/8+2);
     t.right(92);
     t.forward(2*d);
     for (int i=0; i<4; i++){
         t.left(45);
         t.forward(d/4);
     }
     for (int i=0; i<4; i++){
         t.right(45);
         t.forward(d/4);
     }
     t.forward(3*d);
     for (int i=0; i<4; i++){
         t.right(45);
         t.forward(d/4);
     }
     for (int i=0; i<4; i++){
         t.left(45);
         t.forward(d/4);
     }    
     t.forward(2*d);
}

void A2M3(Terrapin t, int scalefactor){
     //another puzzle-piece line, again arbitrary;
     //make sure not to use any absolute coordinates
     t.setPenColor(0, 0, 0);
     int d=scalefactor;
     strokeWeight(d/8+2);
     int alpha = 15;
     t.right(alpha);
     t.forward(2*d);
     for (int i=0; i<2; i++){
         t.left(90);
         t.forward(d/2);
     }
     for (int i=0; i<2; i++){
         t.right(90);
         t.forward(d/2);
     }
     t.forward(2*d);
     for (int i=0; i<2; i++){
         t.right(90);
         t.forward(d/2);
     }
     for (int i=0; i<2; i++){
         t.left(90);
         t.forward(d/2);
     }    
     t.forward(2*d);
     t.left(alpha);
}

void C2M2(Terrapin t, int scalefactor, PVector M){
     // parameter r = -1 means reflected
     //a kind of zigsaw, somewhat arbitrary;
     //make sure not to use any absolute coordinates;
     //two versions of this make the C line AB around M;
     t.setPenColor(0,0, 0);
     int d = scalefactor;
     //int theta = 127;
     strokeWeight(d/8+2);
     t.right(135);
     t.forward(3*d+d/2);
     t.left(100);
     for (int i=0; i<3; i++){
         t.forward(d+d/2);
         t.right(150);
         t.forward(d+d/2);
         t.left(120);
     }   
     //t.right(10);
     //t.forward(3*d);
     PPoint here = getPosition(t);
     float dis = PVector.dist(here,M);
     t.moveToward(int(M.x),int(M.y),int(dis));
     //that is shoot the final stretch towards M 
     //it will not scale well, but no loose ends   
}

void I2H(Terrapin t, int scalefactor){
     // this will be a dashed line;
     //not part of the figure, only for explanation
     t.setPenColor(255,0, 0);
     int d=scalefactor;
     strokeWeight(d/8+2);
     t.drawing=false;
     t.forward(d/2);
     for (int i=0; i<9; i++){
         t.drawing=true;
         t.forward(d);
         t.drawing=false;
         t.forward(d);
     }
     t.drawing=true;
}

void drwIntro(){
     textFont(font32,32);
     text("Nr.3, Basic Type CCC",200,50);
     textFont(font16,16);
     text("Let AB be a C-line. Draw from A another, and from B a third C-line towards the point C, which is chosen freely (but not lying on the straight line through A and B)."
         , 200, 100,700,200); 
     text("Network 666,"
         , 650, 200,700,100); 
     text("2 positionings."
         , 650, 220,700,100); 
}
void drwArrowFull(Terrapin t, int scalefactor){
     //draw a big full arrow
     //s must be prferably arround 20
     t.setPenColor(255 , 0, 0);
     int s=scalefactor;
     strokeWeight(s/4+1);
     //outer arrow
     t.forward(3*s);
     t.left(90);
     t.forward(s/4);
     t.right(110);
     t.forward(s);
     t.right(160);
     t.forward(s);//fill tip
     t.forward(-s);//idem
     t.left(20);
     t.forward(s);
     t.right(110);
     t.forward(s/4);
     t.left(90);
     t.forward(3*s);
     t.right(90);
     t.forward(s/8);
}
void drwArrowOpen(Terrapin t, int scalefactor){
     //draw a big non-filled arrow
     //s must be prferably arround 20
     t.setPenColor(255 , 0, 0);
     int s=scalefactor;
     strokeWeight(s/8);
     t.forward(3*s);
     t.left(90);
     t.forward(s/4);
     t.right(110);
     t.forward(s);
     t.right(160);
     t.left(20);
     t.forward(s);
     t.right(110);
     t.forward(s/4);
     t.left(90);
     t.forward(3*s);
     t.right(90);
     t.forward(s/8);
}
void drwA(PVector p, int scalefactor){
     //to mark the points and write their name;
     //(name writing only only for the big versions);
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(255,0,0);
     fill(255,0,0);
     ellipse(p.x,p.y,d+2,d+2); 
     if (d>5) { textFont(font32,32); text("A",p.x-10,p.y-10);}
}

void drwB(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(255,0,0);
     fill(255,0,0);
     ellipse(p.x,p.y,d+2,d+2); 
     if (d>5) { textFont(font32,32);text("B",p.x,p.y+35); }
}

void drwC(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(255,0,0);
     fill(255,0,0);
     ellipse(p.x,p.y,d+2,d+2); 
     if (d>5) { textFont(font32,32); text("C",p.x-10,p.y-10);}
}
void drwH(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {
        ellipse(p.x,p.y,d+2,d+2); 
        textFont(font32,32);
        text("H",p.x+10,p.y+10);
     }
}  

void drwI(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {
         ellipse(p.x,p.y,d+2,d+2); 
         textFont(font32,32);
         text("I",p.x+10,p.y+10);
      }  
}
     
void drwM(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     ellipse(p.x,p.y,d,d); 
     fill(255,255,255);
     ellipse(p.x,p.y,d+2,d+2); 
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {textFont(font32,32); text("M",p.x-30,p.y+35); } 
}

void drwM1(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     ellipse(p.x,p.y,d,d); 
     fill(255,255,255);
     ellipse(p.x,p.y,d+2,d+2); 
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {textFont(font32,32); text("M1",p.x-55,p.y+5); } 
}
void drwM2(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     ellipse(p.x,p.y,d,d); 
     fill(255,255,255);
     ellipse(p.x,p.y,d+2,d+2); 
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {textFont(font32,32); text("M2",p.x,p.y+35); } 
}
void drwM3(PVector p, int scalefactor){
     int d=scalefactor/2;
     strokeWeight(d/4+2);
     stroke(0,0,255);
     fill(0,0,255);
     ellipse(p.x,p.y,d,d); 
     fill(255,255,255);
     ellipse(p.x,p.y,d+2,d+2); 
     stroke(0,0,255);
     fill(0,0,255);
     if (d>5) {textFont(font32,32); text("M3",p.x-20,p.y+35); } 
}

void rotate2D(PVector v, float theta) {
     // Rotate a vector in 2D
     // Hmm, why doesn't the PVector rotate function work ??? see
     // www.shiffman.net/2011/02/03/rotate-a-vector-processing-js/
     // What's the magnitude?
     float m = v.mag();
     // What's the angle?
     float a = v.heading2D();
     // Change the angle
     a += theta;
     // Polar to cartesian for the new xy components
     v.x = m * cos(a);
     v.y = m * sin(a);
}

void stop(){
     endRecord();
}



